pub fn part_one(input: &str) -> Option<u32> {
    let elf_calories: Vec<Vec<u32>> = input.split("\n\n").
        map(|elf_lines| elf_lines.split("\n").
        map(|calorie_line| calorie_line.parse::<u32>().unwrap()).
        collect())
        .collect();

    let max_calories = elf_calories.into_iter()
        .map(|single_elf_cals| single_elf_cals.into_iter()
        .fold(0,|acc, item| acc + item))
        .fold(0, |acc, item| if item > acc { item } else { acc });
    
    Some(max_calories)
}

pub fn part_two(input: &str) -> Option<u32> {
    let mut elf_calories: Vec<u32> = input.split("\n\n")
    .map(|elf_lines| elf_lines.split("\n")
    .map(|calorie_line| calorie_line.parse::<u32>().unwrap())
    .fold(0, |acc, item| acc + item))
    .collect();

    elf_calories.sort(); // not functional, but I don't feel like doing weird ring stuff

    Some(elf_calories.iter().rev().take(3).sum())
}

fn main() {
    let input = &advent_of_code::read_file("inputs", 1);
    advent_of_code::solve!(1, part_one, input);
    advent_of_code::solve!(2, part_two, input);
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_part_one() {
        let input = advent_of_code::read_file("examples", 1);
        assert_eq!(part_one(&input), Some(24000));
    }

    #[test]
    fn test_part_two() {
        let input = advent_of_code::read_file("examples", 1);
        assert_eq!(part_two(&input), Some(206152));
    }
}
