#[derive(PartialEq, Clone, Copy, Debug)]
enum RPS {
    Rock,
    Paper,
    Scissors
}


impl RPS {
    fn victim(&self) -> RPS {
        match self {
            RPS::Rock => RPS::Scissors,
            RPS::Paper => RPS::Rock,
            RPS::Scissors => RPS::Paper
        }
    }

    fn killer(&self) -> RPS {
        match self {
            RPS::Rock => RPS::Paper,
            RPS::Paper => RPS::Scissors,
            RPS::Scissors => RPS::Rock
        }
    }

    fn shape_score(&self) -> u32 {
        match self {
            RPS::Rock => 1,
            RPS::Paper => 2,
            RPS::Scissors => 3
        }
    }

    fn from_char(c: char) -> RPS {
        match c {
            'A' => RPS::Rock,
            'X' => RPS::Rock,
            'B' => RPS::Paper,
            'Y' => RPS::Paper,
            'C' => RPS::Scissors,
            'Z' => RPS::Scissors,
            _ => panic!("wtf is a {c}")
        }
    }
}

fn score_hand(left: &RPS, right: &RPS) -> u32 {
    if left.victim() == *right {
        return 0;
    }

    if right.victim() == *left {
        return 6;
    }

    return 3;
}

fn score_line(first: &RPS, second: &RPS) -> u32 {
    score_hand(first, second) + second.shape_score()
}

pub fn part_one(input: &str) -> Option<u32> {
    let result: u32 = input.split("\n").map(|item| 
        score_line(
            &RPS::from_char(item.chars().nth(0).unwrap()),
            &RPS::from_char(item.chars().nth_back(0).unwrap())
        )).sum();

    Some(result)
}

pub fn part_two(input: &str) -> Option<u32> {
    let mut score = 0;
    for line in input.split("\n") {
        let op_hand = &RPS::from_char(line.chars().nth(0).unwrap());
        let our_hand = match line.chars().nth_back(0).unwrap() {
            'X' => op_hand.victim(),
            'Y' => *op_hand,
            'Z' => op_hand.killer(),
            _ => panic!("dunno how to do that")
        };
        let this_score = score_line(op_hand, &our_hand);
        score += this_score;
    }

    Some(score)
}

fn main() {
    let input = &advent_of_code::read_file("inputs", 2);
    advent_of_code::solve!(1, part_one, input);
    advent_of_code::solve!(2, part_two, input);
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_part_one() {
        let input = advent_of_code::read_file("examples", 2);
        assert_eq!(part_one(&input), Some(15));
    }

    #[test]
    fn test_part_two() {
        let input = advent_of_code::read_file("examples", 2);
        assert_eq!(part_two(&input), Some(12));
    }
}
