#![feature(iter_array_chunks)]
use std::{collections::HashSet};

trait Priority {
    fn priority(&self) -> u32;
}

impl Priority for char {
    fn priority(&self) -> u32 {
        let our_val: u32 = (*self).into();

        if 'a' <= *self && *self <= 'z' {
            let offset_val: u32 = 'a'.into();
            return our_val - offset_val + 1;
         } else {
            let offset_val: u32 = 'A'.into();
            return our_val - offset_val + 27;
         }
    }
}


pub fn part_one(input: &str) -> Option<u32> {
    let mut priority_sum: u32 = 0;

    for line in input.split("\n") {
        let mut rucksack = HashSet::new();
        let cap = line.len() / 2;

        for idx in 0..cap {
            rucksack.insert(line.chars().nth(idx).unwrap());
        }

        for idx in cap..cap*2 {
            let item_type = line.chars().nth(idx).unwrap();
            if rucksack.contains(&item_type) {
                priority_sum += item_type.priority();
                break;
            }
        }
    }

    Some(priority_sum)
}

fn find_badge_for_group(first: &str, second: &str, third: &str) -> char {
    // I felt really cool when I wrote this but it also just looks goddamn insane now
    // tldr, build hashsets from each line, intersect them all, assume there's only one thing left in them (b.c. problem spec) and yoink that out
    [first, second, third].iter().map(|line| HashSet::from_iter(line.chars()))
        .reduce(|acc: HashSet<char>, new_hs: HashSet<char>| HashSet::from_iter(acc.intersection(&new_hs).copied()))
        .unwrap().iter().next().unwrap().clone()
}

pub fn part_two(input: &str) -> Option<u32> {
    Some(input.split("\n").array_chunks::<3>()
        .map(|item| find_badge_for_group(item[0], item[1], item[2]))
        .map(|item| item.priority())
        .sum())
}

fn main() {
    let input = &advent_of_code::read_file("inputs", 3);
    advent_of_code::solve!(1, part_one, input);
    advent_of_code::solve!(2, part_two, input);
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_part_one() {
        let input = advent_of_code::read_file("examples", 3);
        assert_eq!(part_one(&input), Some(157));
    }

    #[test]
    fn test_part_two() {
        let input = advent_of_code::read_file("examples", 3);
        assert_eq!(part_two(&input), Some(70));
    }
}
