use std::{str::FromStr, mem::swap};

struct Pair {
    first: u32,
    second: u32
}

impl FromStr for Pair {
    type Err = std::string::ParseError;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let (x, y) = s.split_once("-").unwrap();
        return Ok(
            Pair{
                first: x.parse().unwrap(),
                second: y.parse().unwrap()
            }
        )
    }
}

fn parse_line(line: &str) -> (Pair, Pair) {
    let (first_str, second_str) = line.split_once(",").unwrap();
    let mut p1 = Pair::from_str(first_str).unwrap();
    let mut p2 = Pair::from_str(second_str).unwrap();

    // ensure p1 always "comes first" 
    if p1.first > p2.first {
        swap(&mut p1, &mut p2);
    } else if p1.first == p2.first && p1.second < p2.second {
        swap(&mut p1, &mut p2);
    }

    (p1, p2)
}

pub fn part_one(input: &str) -> Option<u32> {
    Some(input.split("\n")
        .map(|line| parse_line(line))
        .fold(0, |count, (p1, p2)| if p1.first <= p2.first && p2.second <= p1.second { count + 1} else {count}))
}

pub fn part_two(input: &str) -> Option<u32> {
    Some(input.split("\n")
        .map(|line| parse_line(line))
        .fold(0, |count, (p1, p2)| if p1.first <= p2.first && p2.first <= p1.second { count + 1} else {count}))
}

fn main() {
    let input = &advent_of_code::read_file("inputs", 4);
    advent_of_code::solve!(1, part_one, input);
    advent_of_code::solve!(2, part_two, input);
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_part_one() {
        let input = advent_of_code::read_file("examples", 4);
        assert_eq!(part_one(&input), Some(2));
    }

    #[test]
    fn test_part_two() {
        let input = advent_of_code::read_file("examples", 4);
        assert_eq!(part_two(&input), Some(4));
    }
}
