#![feature(get_many_mut)]
pub fn parse_stack(input: Vec<&str>, num_stacks: i32) -> Vec<Vec<char>> {
    let mut stacks: Vec<Vec<char>> = (0..num_stacks).map(|_| Vec::new()).collect();

    for line in input {
        let mut linewalker = line.chars().enumerate();
        linewalker.next(); // skip first
        while let Some((idx, c)) = linewalker.next() {
            if c == ' ' {
                // do nothing
            } else {
                stacks[(idx - 1) / 4].push(c);
            }
            linewalker.nth(2); // consume (total) four items per loop cycle
        }
    }

    for stack in stacks.iter_mut() { stack.reverse();}
    stacks.to_owned()
}

struct Move {
    from: usize,
    to: usize,
    qty: u32
}

fn parse_input(input: &str) -> (Vec<Vec<char>>, Vec<Move>) {
    let mut lines = input.split("\n");
    let mut num_lines = -1;

    let mut initial_state: Vec<&str> = Vec::new();
    while let Some(line) = lines.next() {
        if line.contains('[') {
            initial_state.push(line);
        } else {
            // get the number of stacks
            let thing = line.trim().split(" ").last().unwrap();
            num_lines = thing.parse().unwrap();
            break;
        }
    }

    lines.next(); // skip the blank line

    let stacks = parse_stack(initial_state, num_lines);

    let moves = lines
        .map(|line| line.split(" ").collect())
        .map(|chunks: Vec<&str>| Move {
            qty: chunks[1].parse().unwrap(),
            from: chunks[3].parse().unwrap(),
            to: chunks[5].parse().unwrap()
        }).collect();
    
    (stacks, moves)
}

pub fn part_one(input: &str) -> Option<String> {
    let (mut stacks, moves) = parse_input(input);
    
    for mv in moves {
        for _ in 0..mv.qty {
            let crane_crate = stacks.get_mut(mv.from - 1).unwrap().pop().unwrap();
            stacks.get_mut(mv.to - 1).unwrap().push(crane_crate);
        }   
    }

    Some(stacks.iter().map(|stack| stack.iter().rev().next().unwrap()).collect())
}

fn move_n_crates(from: &mut Vec<char>, to: &mut Vec<char>, n: u32) {
    let mut tmp = Vec::new();
    for _ in 0..n {
        tmp.push(from.pop().unwrap());
    }

    for _ in 0..n {
        to.push(tmp.pop().unwrap());
    }
}

pub fn part_two(input: &str) -> Option<String> {
    let (mut stacks, moves) = parse_input(input);
    
    for mv in moves {
        let [from, to] = stacks.get_many_mut([mv.from - 1, mv.to - 1]).unwrap();
        move_n_crates(from, to, mv.qty);
    }

    Some(stacks.iter().map(|stack| stack.iter().rev().next().unwrap()).collect())
}

fn main() {
    let input = &advent_of_code::read_file("inputs", 5);

    // note: lines iterator should be on the blank between initial and directions
    advent_of_code::solve!(1, part_one, input);
    advent_of_code::solve!(2, part_two, input);
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_part_one() {
        let input = advent_of_code::read_file("examples", 5);
        assert_eq!(part_one(&input), Some("CMZ".to_string()));
    }

    #[test]
    fn test_part_two() {
        let input = advent_of_code::read_file("examples", 5);
        assert_eq!(part_two(&input), Some("MCD".to_string()));
    }
}
