use std::{collections::HashSet};

pub fn part_one(input: &str) -> Option<u32> {
    let chars = input.chars();
    let mut packet: [char; 4] = [' ', ' ', ' ', ' '];

    for (idx, c) in chars.enumerate() {
        packet[idx % 4] = c;
        let acc: HashSet<&char> = HashSet::from_iter(packet.iter());
        if idx > 3 && acc.len() == 4 {
            return Some((idx + 1).try_into().unwrap());
        }
    }
    None
}

pub fn part_two(input: &str) -> Option<u32> {
    const WINDOW_SIZE: usize = 14;
    let chars = input.chars();
    let mut packet: [char; WINDOW_SIZE] = [' '; WINDOW_SIZE];

    for (idx, c) in chars.enumerate() {
        packet[idx % WINDOW_SIZE] = c;
        let acc: HashSet<&char> = HashSet::from_iter(packet.iter());
        if idx >= WINDOW_SIZE && acc.len() == WINDOW_SIZE {
            return Some((idx + 1).try_into().unwrap());
        }
    }
    None
}

fn main() {
    let input = &advent_of_code::read_file("inputs", 6);
    advent_of_code::solve!(1, part_one, input);
    advent_of_code::solve!(2, part_two, input);
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_part_one() {
        let input = advent_of_code::read_file("examples", 6);
        assert_eq!(part_one(&input), Some(5));
    }

    #[test]
    fn test_part_two() {
        let input = advent_of_code::read_file("examples", 6);
        assert_eq!(part_two(&input), Some(19));
    }
}
